/// <reference types="node" />
import type { ContractClientOptions, Tx } from "./types";
import { SorobanRpc } from "..";
import { AssembledTransaction } from "./assembled_transaction";
/**
 * A transaction that has been sent to the Soroban network. This happens in two steps:
 *
 * 1. `sendTransaction`: initial submission of the transaction to the network.
 *    If this step runs into problems, the attempt to sign and send will be
 *    aborted. You can see the result of this call in the
 *    `sendTransactionResponse` getter.
 * 2. `getTransaction`: once the transaction has been submitted to the network
 *    successfully, you need to wait for it to finalize to get the result of the
 *    transaction. This will be retried with exponential backoff for
 *    {@link MethodOptions.timeoutInSeconds} seconds. See all attempts in
 *    `getTransactionResponseAll` and the most recent attempt in
 *    `getTransactionResponse`.
 */
export declare class SentTransaction<T> {
    signTransaction: ContractClientOptions["signTransaction"];
    assembled: AssembledTransaction<T>;
    server: SorobanRpc.Server;
    signed?: Tx;
    /**
     * The result of calling `sendTransaction` to broadcast the transaction to the
     * network.
     */
    sendTransactionResponse?: SorobanRpc.Api.SendTransactionResponse;
    /**
     * If `sendTransaction` completes successfully (which means it has `status: 'PENDING'`),
     * then `getTransaction` will be called in a loop for
     * {@link MethodOptions.timeoutInSeconds} seconds. This array contains all
     * the results of those calls.
     */
    getTransactionResponseAll?: SorobanRpc.Api.GetTransactionResponse[];
    /**
     * The most recent result of calling `getTransaction`, from the
     * `getTransactionResponseAll` array.
     */
    getTransactionResponse?: SorobanRpc.Api.GetTransactionResponse;
    static Errors: {
        SendFailed: {
            new (message?: string | undefined): {
                name: string;
                message: string;
                stack?: string | undefined;
            };
            captureStackTrace(targetObject: object, constructorOpt?: Function | undefined): void;
            prepareStackTrace?: ((err: Error, stackTraces: NodeJS.CallSite[]) => any) | undefined;
            stackTraceLimit: number;
        };
        SendResultOnly: {
            new (message?: string | undefined): {
                name: string;
                message: string;
                stack?: string | undefined;
            };
            captureStackTrace(targetObject: object, constructorOpt?: Function | undefined): void;
            prepareStackTrace?: ((err: Error, stackTraces: NodeJS.CallSite[]) => any) | undefined;
            stackTraceLimit: number;
        };
    };
    constructor(signTransaction: ContractClientOptions["signTransaction"], assembled: AssembledTransaction<T>);
    /**
     * Initialize a `SentTransaction` from an existing `AssembledTransaction` and
     * a `signTransaction` function. This will also send the transaction to the
     * network.
     */
    static init: <T_1>(signTransaction: ContractClientOptions["signTransaction"], assembled: AssembledTransaction<T_1>) => Promise<SentTransaction<T_1>>;
    private send;
    get result(): T;
}
